import ctypes
import os

from .utils import get_dependency_filename

# Load the compiled miniz
root_dir = os.path.abspath(os.path.dirname(__file__))
library = ctypes.cdll.LoadLibrary(f'{root_dir}/dependencies/{get_dependency_filename()}')

# Define function signatures for decompression and compression
library.mz_compress2.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_ulong), ctypes.c_void_p, ctypes.c_ulong, ctypes.c_int]
library.mz_compress2.restype = ctypes.c_int

library.mz_uncompress.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_ulong), ctypes.c_void_p, ctypes.c_ulong]
library.mz_uncompress.restype = ctypes.c_int


class Miniz:
    # Decompress using miniz
    def decompress(data):
        source = ctypes.create_string_buffer(data)
        source_len = len(data)
        dest_len = ctypes.c_ulong(source_len * 4)  # Allocate more space for decompressed data
        dest = ctypes.create_string_buffer(dest_len.value)

        result = library.mz_uncompress(dest, ctypes.byref(dest_len), source, source_len)
        if result != 0:
            raise RuntimeError("Decompression failed")
        return dest.raw[:dest_len.value]


    # Compress using miniz with the default compression level
    def compress(data, level=1):
        source = ctypes.create_string_buffer(data)
        source_len = len(data)
        dest_len = ctypes.c_ulong(source_len * 2)  # Allocate more space for compressed data
        dest = ctypes.create_string_buffer(dest_len.value)

        result = library.mz_compress2(dest, ctypes.byref(dest_len), source, source_len, level)
        if result != 0:
            raise RuntimeError("Compression failed")
        return dest.raw[:dest_len.value]


from sys        import platform
from platform   import machine

import ctypes


def get_dependency_filename():
    if platform == 'darwin':
        file_ext = '-macOS-arm64.dylib' if machine() == "arm64" else '-macOS-x86_64.dylib'
    elif platform in ('win32', 'cygwin'):
        file_ext = '-Windows-arm64.dll' if 8 == ctypes.sizeof(ctypes.c_voidp) else '-Windows-x86_64.dll'
    else:
        if machine() == "aarch64":
            file_ext = '-Linux-arm64.so'
        elif "x86" in machine():
            file_ext = '-Linux-x86_64.so'
        else:
            file_ext = '-Linux-arm64.so'

    return f'miniz{file_ext}'